#include <Servo.h>
#define ECHOPIN     7    // Echo Pin
#define TRIGPIN     8    // Trigger Pin
#define PINGMIN     0    // Minimal-Entfernung
#define PINGMAX    80    // Maximal-Entfernung
#define SERVOPIN    9    // Servo Pin
#define ANGLESTART  0    // Startwinkel, ggf. anpassen
#define ANGLESTOP 180    // Stopwinkel, ggf. anpassen
#define DELAYSERVO 10    // Pausenwert für Servo

Servo meinServo;         // Servo-Objekt
int angle;               // Aktueller Servo-Bewegungswinkel
int movement = 1;        // 1: rechts, -1: links
int dist = 0;            // Gemessene Entfernung


int getPing(int minV, int maxV) {
  long entfernung = 0, zeit = 0;
  digitalWrite(TRIGPIN, LOW);
  delayMicroseconds(3);
  digitalWrite(TRIGPIN, HIGH); // Trigger Impuls 10 us
  delayMicroseconds(10);
  digitalWrite(TRIGPIN, LOW);
  zeit = pulseIn(ECHOPIN, HIGH); // Die Echo-Zeit messen
  zeit = (zeit/2);               // Die Zeit halbieren
  entfernung = zeit / 29.1;      // Die Zeit in Zentimeter umrechnen
  if((entfernung < minV) or (entfernung > maxV)) entfernung = -1;
  return entfernung;            // Entfernung zurückgeben
}

void setup() {
  meinServo.attach(SERVOPIN); // Objekt mit Servo-Pin verbinden
  Serial.begin(9600);         // Serielle Schnittstelle initialisieren  
  angle = 90;                 // Winkel = 90°
  meinServo.write(angle);     // Servo-Winkel in Mittelstellung
  delay(2000);                // 2 Sekunden warten

  pinMode(TRIGPIN, OUTPUT);   // Pin als Ausgang
  pinMode(ECHOPIN, INPUT);    // Pin als Eingang
}

void loop() {
  angle = angle + movement; // Bewegungswinkel berechnen
  if(angle >= ANGLESTOP)    // Wennn > Stopwinkel => Richtungswechsel          
    movement = -1;
  if(angle == ANGLESTART)   // Wennn = Startwinkel => Richtungswechsel
    movement = 1;
  meinServo.write(angle);   // Servo ansteuern  
  dist = getPing(PINGMIN, PINGMAX); // Momentane Entfernung ermitteln
  // Versende-String an COM-Port
  String s = String(angle) + ";" + String(dist); 
  Serial.println(s);      // Ausgabe des Wertes an COM
  delay(DELAYSERVO);      // Eine kurze Pause
}
