/*
Author:        Erik Bartmann
Website:       http://www.erik-bartmann.de
Blog:          http://www.tumblr.com/tumblelog/arduino-processing
Twitter:       https://twitter.com/#!/ErikBartmann
Flickr:        http://www.flickr.com/photos/erikbartmann/6317487333 (Schaltungsaufbau)
Scope:         Das Zahlenratespiel mit LC-Display (Analog)
Version:       1.0
Date:          03.10.2011
Last modified:
Description:   Dieser Sketch zeigt Dir die Ansteuerung eines KeyPads inklusive LC-Display
               anhand eines Zahlenratespiels
Referenz:      http://www.arduino.cc/en/Reference/LiquidCrystal
               http://arduino.cc/en/Reference/If
               http://arduino.cc/en/Reference/Else
               http://arduino.cc/en/Reference/Delay
               http://arduino.cc/en/Reference/AnalogRead
               http://arduino.cc/en/Reference/Random
               http://arduino.cc/en/Reference/RandomSeed
*/
#include <LiquidCrystal.h>
#include "MyAnalogKeyPad.h"

#define analogPinKeyPad A0 // Definition des analogen Pins
#define MIN 10            // Untergrenze für Zufallszahl
#define MAX 1000          // Obergrenze für Zufallszahl
#define RS 12             // LCD-Register Select Pin
#define E 11              // LCD-Enable Pin
#define D4 5              // LCD-Datenleitung Pin 4
#define D5 4              // LCD-Datenleitung Pin 5
#define D6 3              // LCD-Datenleitung Pin 6
#define D7 2              // LCD-Datenleitung Pin 7
#define COLS 16           // Anzahl der LCD-Spalten
#define ROWS 2            // Anzahl der LCD-Zeilen

int arduinoZahl, versuche; // Die generierte Zahl, Anzahl der Versuche
char deineZahl[5];         // Max. 5-stellige Zahl
byte stelle;
MyAnalogKeyPad myOwnKeyPad(analogPinKeyPad); // KeyPad Instanziierung
LiquidCrystal lcd(RS, E, D4, D5, D6, D7);    // LCD Instanziierung

void setup(){
  myOwnKeyPad.setDebounceTime(500); // Prellzeit auf 500ms setzen
  lcd.begin(COLS, ROWS);            // Anzahl der Spalten und Zeilen
  lcd.blink();                      // Cursor blinken lassen
  startSequence();                  // Aufruf der Startsequenz
}

void loop(){
  char myKey = myOwnKeyPad.readKey(); // Abfragen des gedrückten Tasters
  if(myKey != KEY_NOT_PRESSED){ // Abfrage, ob irgendein Taster gedrückt
    deineZahl[stelle] = myKey;
    stelle++;
    lcd.print(myKey); // Taste im LCD anzeigen
  }
  if(stelle == int(log10(MAX))+1){
    versuche++;
    int a = atoi(deineZahl);
    if(a == arduinoZahl){
      lcd.clear();             // LCD-Anzeige löschen
      lcd.print("Erraten!!!"); // Ausgabe an das LCD
      lcd.setCursor(0, 1);     // Cursor in die 2.Zeile positionieren
      lcd.print("Versuche: " + String(versuche));
      delay(4000);     // 4 Sekunden warten
      versuche = 0;    // Anzahl der Versuche zurücksetzen
      startSequence(); // Startsequenz aufrufen
    }
    else if(a < arduinoZahl){
      lcd.setCursor(0, 1);   // Cursor in die 2.Zeile positionieren
      lcd.print("Zu klein"); // Ausgabe an das LCD
      lcd.setCursor(0, 0);   // Cursor in die 1.Zeile positionieren
    }
    else{
      lcd.setCursor(0, 1);   // Cursor in die 2.Zeile positionieren
      lcd.print("Zu gross"); // Ausgabe an das LCD
      lcd.setCursor(0, 0);   // Cursor in die 1.Zeile positionieren
    }
    lcd.setCursor(2, 0); // Cursor an die 3.Stelle der 1.Zeile positionieren
    stelle = 0;
  }
}

int zufallszahl(int minimum, int maximum){
  randomSeed(analogRead(5));
  return random(minimum, maximum + 1);
}

void startSequence(){
  arduinoZahl = zufallszahl(MIN, MAX); // Zu erratende Zahl generieren
  lcd.clear(); // LCD-Anzeige löschen
  lcd.print("Rate eine Zahl"); // Ausgabe an das LCD
  lcd.setCursor(0, 1);         // Cursor in die 2.Zeile positionieren
  lcd.print("von " + String(MIN) + " - " + String(MAX));
  delay(4000);     // 4 Sekunden warten
  lcd.clear();     // LCD-Anzeige löschen
  lcd.print(">>"); // Ausgabe an das LCD
}
