﻿/*
 * Author:      Erik Bartmann
 * Scope:       Arduino-Talker
 * Date:        27.10.2016
 * Modified:    27.16.2016
 * Internet:    www.erik-bartmann.de
 * Message:     Good luck, have fun!
 * */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ArduinoTalker
{
    public partial class frmMain : Form
    {
        #region field(s)
        private frmArduinoSketchCode myArduinoSketchCode = new frmArduinoSketchCode(); // Form for sketch-code
        private SerialPort SerialPort; // COM-Object
        private string COMPort;        // COM-Port String
        private int BaudRate;          // Baud-Rate-String
        private int COMPortCount;      // Number of available COM-Ports
        private char CR = (char)13;    // Carriage-Return character
        #endregion

        public frmMain()
        {
            InitializeComponent(); // Init Components
            initEventHandler();    // Init Event-Handler
        }

        private void initEventHandler()
        {
            foreach (Control c in groupBoxMain.Controls)
            {
                if (c is Button)
                    (c as Button).Click += new EventHandler(PinButton_Click);
                if (c is HScrollBar)
                    (c as ScrollBar).Scroll += new ScrollEventHandler(ScrollBar_Scroll);
                comboBoxCOMPort.SelectedIndexChanged += new EventHandler(comboBoxCOM_SelectedIndexChanged);
                comboBoxBaudRate.SelectedIndexChanged += new EventHandler(comboBoxCOM_SelectedIndexChanged);
            }
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            Text = Variables.APPLICATIONNAME + " " + Variables.APPLICATIONVERSION + " by " + Variables.AUTHORNAME;
            LinkLabel.Link websitelink = new LinkLabel.Link();
            websitelink.LinkData = Variables.LINKWEBSITE;
            linkLabelWebsite.Text = Variables.LINKWEBSITE;
            linkLabelWebsite.Links.Add(websitelink);
            LinkLabel.Link emaillink = new LinkLabel.Link();
            emaillink.LinkData = Variables.LINKEMAIL;
            linkLabelEmail.Text = Variables.LINKEMAIL;
            linkLabelEmail.Links.Add(emaillink);
            foreach (string s in SerialPort.GetPortNames())
            {
                COMPortCount++;
                comboBoxCOMPort.Items.Add(s);
                comboBoxCOMPort.Sorted = true;
            }
            if (COMPortCount == 0)
            {
                panelMain.Enabled = false;
                toolStripStatusLabelMessage.Text = "No COMPort availabe!";
            }
            comboBoxBaudRate.Items.AddRange(new string[]{"300", "1200", "2400",
                                                         "4800", "9600", "14400",
                                                         "19200", "28800", "38400",
                                                         "57600", "115200"});
            btnConnect.Enabled = false;
            groupBoxMain.Enabled = false;
            statusStripMain.ImageList = imageListCOMStatus;
            SerialPort = new SerialPort();
        }

        private void linkLabelWebsite_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start(e.Link.LinkData.ToString());
            linkLabelWebsite.LinkColor = Color.Black;
            linkLabelWebsite.LinkVisited = true;
        }

        private void linkLabelEmail_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("mailto:" + linkLabelEmail.Text);
        }

        private void Send(string[] a)
        {
            // Write string(s)
            foreach (string s in a)
            {
                SerialPort.Write(s + CR + CR);
                toolStripStatusLabelSendString.Text = s;
            }
            if ((a.Length > 1) && (a[0].IndexOf("D1") !=-1))
                toolStripStatusLabelSendString.Text = "All on";
            if ((a.Length > 1) && (a[0].IndexOf("D0") != -1))
                toolStripStatusLabelSendString.Text = "All off";
        }

        void comboBoxCOM_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBoxCOMPort.Text != "")
                COMPort = comboBoxCOMPort.Text;
            if (comboBoxBaudRate.Text != "")
                BaudRate = Convert.ToInt32(comboBoxBaudRate.Text);
            btnConnect.Enabled = (comboBoxCOMPort.Text != "" && comboBoxBaudRate.Text != "");
            toolTipButtons.SetToolTip(this.btnConnect, "Connect to selected COM-Port: " + comboBoxCOMPort.Text);
        }

        void PinButton_Click(object sender, EventArgs e)
        {
            string sendString = string.Empty;
            string buttonName = ((sender as Button).Name);
            string pin = string.Empty;
            // On pressed
            if ((buttonName.IndexOf("On") > -1) && (buttonName != "btnPinAllOn"))
            {
                pin = buttonName.Substring(6, 2);
                sendString = "S" + pin + "D1";
                toolStripStatusLabelMessage.Text = "Pin " + pin + " set to HIGH-Level";
                Send(new String[] { sendString });
            }

            // Off pressed
            if ((buttonName.IndexOf("Off") > -1) && (buttonName != "btnPinAllOff"))
            {
                pin = buttonName.Substring(6, 2);
                sendString = "S" + pin + "D0";
                toolStripStatusLabelMessage.Text = "Pin " + pin + " set to LOW-Level";
                Send(new String[] { sendString });
            }

            // Blink pressed
            if (buttonName.IndexOf("Blink") > -1)
            {
                pin = buttonName.Substring(6, 2);
                sendString = "S" + pin + "B1";
                toolStripStatusLabelMessage.Text = "Pin " + pin + " set to Blink-Level";
                Send(new String[] { sendString });
            }

            // All on pressed
            if (buttonName == "btnPinAllOn")
            {
                Send(new String[] { "S2D1", "S3D1", "S4D1", "S5D1",
                                    "S6D1", "S7D1", "S8D1", "S9D1",
                                    "S10D1", "S11D1", "S12D1", "S13D1"});
                toolStripStatusLabelMessage.Text = "All pins set to HIGH-Level";
            }

            // All off pressed
            if (buttonName == "btnPinAllOff")
            {
                Send(new String[] { "S2D0", "S3D0", "S4D0", "S5D0",
                                    "S6D0", "S7D0", "S8D0", "S9D0",
                                    "S10D0", "S11D0", "S12D0", "S13D0"});
                toolStripStatusLabelMessage.Text = "All pins set to LOW-Level";
            }
        }

        void ScrollBar_Scroll(object sender, ScrollEventArgs e)
        {
            string sendString = string.Empty;
            if (sender == hScrollBarPort03)
            {
                sendString = "S3A" + hScrollBarPort03.Value.ToString("0000");
                lblPWMPort03.Text = hScrollBarPort03.Value.ToString();
                toolTipScrollbars.SetToolTip(hScrollBarPort03, hScrollBarPort03.Value.ToString());
                toolStripStatusLabelMessage.Text = "Port 03 set to PWM-Level of " + hScrollBarPort03.Value.ToString();
            }
            if (sender == hScrollBarPort05)
            {
                sendString = "S5A" + hScrollBarPort05.Value.ToString("0000");
                lblPWMPort05.Text = hScrollBarPort05.Value.ToString();
                toolTipScrollbars.SetToolTip(hScrollBarPort05, hScrollBarPort05.Value.ToString());
                toolStripStatusLabelMessage.Text = "Port 05 set to PWM-Level of " + hScrollBarPort05.Value.ToString();
            }
            if (sender == hScrollBarPort06)
            {
                sendString = "S6A" + hScrollBarPort06.Value.ToString("0000");
                lblPWMPort06.Text = hScrollBarPort06.Value.ToString();
                toolTipScrollbars.SetToolTip(hScrollBarPort06, hScrollBarPort06.Value.ToString());
                toolStripStatusLabelMessage.Text = "Port 06 set to PWM-Level of " + hScrollBarPort06.Value.ToString();
            }
            if (sender == hScrollBarPort09)
            {
                sendString = "S9A" + hScrollBarPort09.Value.ToString("0000");
                lblPWMPort09.Text = hScrollBarPort09.Value.ToString();
                toolTipScrollbars.SetToolTip(hScrollBarPort09, hScrollBarPort09.Value.ToString());
                toolStripStatusLabelMessage.Text = "Port 09 set to PWM-Level of " + hScrollBarPort09.Value.ToString();
            }
            if (sender == hScrollBarPort10)
            {
                sendString = "S10A" + hScrollBarPort10.Value.ToString("0000");
                lblPWMPort10.Text = hScrollBarPort10.Value.ToString();
                toolTipScrollbars.SetToolTip(hScrollBarPort10, hScrollBarPort10.Value.ToString());
                toolStripStatusLabelMessage.Text = "Port 10 set to PWM-Level of " + hScrollBarPort10.Value.ToString();
            }
            if (sender == hScrollBarPort11)
            {
                sendString = "S11A" + hScrollBarPort11.Value.ToString("0000");
                lblPWMPort11.Text = hScrollBarPort11.Value.ToString();
                toolTipScrollbars.SetToolTip(hScrollBarPort11, hScrollBarPort11.Value.ToString());
                toolStripStatusLabelMessage.Text = "Port 11 set to PWM-Level of " + hScrollBarPort11.Value.ToString();
            }
            // Call Send-Method
            Send(new String[] { sendString });
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            if (!SerialPort.IsOpen)
            {
                btnConnect.Text = "Waiting...";
                Application.DoEvents();
                SerialPort.PortName = COMPort;
                SerialPort.BaudRate = BaudRate;
                SerialPort.DataBits = 8;
                SerialPort.StopBits = StopBits.One;
                SerialPort.Parity = Parity.None;
                try
                {
                    SerialPort.Open();
                    toolStripStatusLabelCOMStatus.ImageIndex = 1; // Green button                    
                    btnConnect.Text = "Disconnect";
                    comboBoxCOMPort.Enabled = false;
                    comboBoxBaudRate.Enabled = false;
                    groupBoxMain.Enabled = true;
                    toolTipButtons.SetToolTip(this.btnConnect, "Disonnect from selected COM-Port: " + comboBoxCOMPort.Text);
                }
                catch (Exception ex)
                {
                    toolStripStatusLabelCOMStatus.ImageIndex = 0; // Red button
                    btnConnect.Text = "Connect";
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                SerialPort.Close();
                btnConnect.Text = "Connect";
                toolStripStatusLabelCOMStatus.ImageIndex = 0; // Red button
                comboBoxCOMPort.Enabled = true;
                comboBoxBaudRate.Enabled = true;
                groupBoxMain.Enabled = false;
                toolTipButtons.SetToolTip(this.btnConnect, "Connect to selected COM-Port: " + comboBoxCOMPort.Text);
            }
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            DialogResult result;
            if (SerialPort.IsOpen)
            {
                result = MessageBox.Show("COM-Port is still open! Close anyway?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
                if (result == System.Windows.Forms.DialogResult.No)
                    e.Cancel = true;
                else
                {
                    if (SerialPort.IsOpen)
                        SerialPort.Close();
                }
            }
        }

        private void btnArduinoSketchCode_Click(object sender, EventArgs e)
        {
            myArduinoSketchCode.ShowDialog();
        }
    }
}
